/**@name LIBRARY: unLaser.ctl

@author: Frdric BERNARD (AB-CO-IS)

Creation Date: 09/02/2004 

Modification History: 

	12/06/2012: Vasileios
  IS-740 :
   - Added const int LASER_T_MODE=13
   - unLaser_Configuration add T_mode
   - unLaser_LoadSettingsManager add T_mode
		- unLaser_setStatusTable add T_mode

	04/04/2007: Herve
		- set LASER_TCP_MAXPACKETSIZE to 10000
		
	07/02/2006: Herve
		- in unLaser_Integrity check if <= instead of ==
		
	10/01/2005 Cline
		- add system name in unLaser_Integrity and unLaser_Check_Integrity

	23/04/2004 Frederic
		- move functions to unCoreLHCServices.ctl

version 1.1

External Functions: 
	
Internal Functions: 
	
	
Purpose: 
This library contains generic functions for LASER management.

Usage: Public

PVSS manager usage: CTRL (WCCOACTRL), UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.

	Functions:
		unLaser_DefaultSettingsManager
		unLaser_LoadSettingsManager
		unLaser_getStatus
		unLaser_setStatusTable
		unLaser_Configuration
		unLaser_getPosition
		unLaser_getFromFile
		unLaser_Check
		unLaser_CheckLine
		unLaser_LoadDpType
		unLaser_Query
		unLaser_Integrity
		unLaser_Update_PVSS_Status
		unLaser_Check_Integrity
		unSystemIntegrity__Laser_CB
		unSystemIntegrity__Laser_CB_Api
*/

//string & time
const int LASER_CONFIGPATH=1;
const int LASER_TCP_LOCALHOST=2;
const int LASER_TCP_REMOTEHOST=3;
const int LASER_TCP_DATAPATH=4;
//const int LASER_NAME=5;

//int & bool
const int LASER_SHOWSTATISTICS=5;
const int LASER_TYPESTATISTICS=6;
const int LASER_TCP_REMOTEPORT=7;
const int LASER_T_MODE=13;
//const int LASER_TCP_TESTMODE=9;
const int LASER_TCP_MAXPACKETSIZE=8;
const int LASER_TCP_TIMEOUT=9;
const int LASER_BACKUP_PERIOD=10;
const int LASER_BUFFER_PERIOD=11;
const int LASER_NUM_PERIOD=12;


//State Archive/Manager
const int Laser_Command=1;
const int Laser_StateOut=2;
const int Laser_Message=3;
const int Laser_PVSS=4;

//Catalogue name
const string g_sCat_Laser="unLaser";

//user allowed to modify LASER if Unicos Package not installed
const string g_Laser_userAdmin="root"; 

//ff constant
const string g_Laser_FF = "0fff";


// unLaser_DefaultSettingsManager
/**
Purpose: set the default settings for Manager DP

Parameter: 	sDP, string, input, DataPoint Name
			exceptionInfoTemp, dyn_string, output, Error message(s)
			
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_DefaultSettingsManager(string sDP, dyn_string &exceptionInfoTemp)
{
int iRes, iTimeout_Alive;
dyn_string dsSettings;
string sFunction, sName;

unLaser_Configuration(dsSettings, sFunction, sName, iTimeout_Alive, exceptionInfoTemp);

if (dynlen(exceptionInfoTemp)==0)
	{
	if (dpExists(sDP + ".Settings"))
		{
		iRes=dpSetWait(sDP + ".Settings.man_ConfigPath", dsSettings[LASER_CONFIGPATH],
				sDP + ".Settings.man_ShowStatistics", dsSettings[LASER_SHOWSTATISTICS],
				sDP + ".Settings.man_BackupPeriod", dsSettings[LASER_BACKUP_PERIOD],
				sDP + ".Settings.tcp_Timeout", dsSettings[LASER_TCP_TIMEOUT],
				sDP + ".Settings.tcp_LocalHost", dsSettings[LASER_TCP_LOCALHOST],
				sDP + ".Settings.tcp_BrokerName", dsSettings[LASER_TCP_REMOTEHOST],
				sDP + ".Settings.tcp_BrokerPort", dsSettings[LASER_TCP_REMOTEPORT],
				sDP + ".Settings.tcp_TopicBase", dsSettings[LASER_TCP_DATAPATH],
				sDP + ".Settings.T_mode", dsSettings[LASER_T_MODE],        
//				sDP + ".Settings.man_Name", dsSettings[LASER_NAME],
	//			sDP + ".Settings.tcp_TestMode", dsSettings[LASER_TCP_TESTMODE],
				sDP + ".Settings.tcp_MaxPacketSize", dsSettings[LASER_TCP_MAXPACKETSIZE],
				sDP + ".Settings.man_NumPeriod", dsSettings[LASER_NUM_PERIOD],
				sDP + ".Settings.man_BufferPeriod", dsSettings[LASER_BUFFER_PERIOD],
				sDP + ".Settings.man_StatisticsType", dsSettings[LASER_TYPESTATISTICS],
				sDP + ".State.st_CommandIn", 100,				//Stop
				sDP + ".State.st_StateOut", 1000,			//Unknow state
				sDP + ".State.ev_MessageOut", c_Ok);		//Unknow message
		}
	else
		fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_DefaultSettingsManager(): " + getCatStr(g_sCat_Laser,"DPDOESNOTEXIST"),"");
	}
}


// unLaser_LoadSettingsManager
/**
Purpose: Get  the settings for Manager

Parameter: 	sDP, string, input, DataPoint Name
			dsParam, dyn string, output, String Parameters
			diParam, dyn int, output, Int Parameters
			exceptionInfoTemp, dyn_string, output, Error message(s)
			
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unLaser_LoadSettingsManager(string sDP, dyn_string &dsParam, dyn_string &exceptionInfoTemp)
{
int iRes, i;

if (dpExists(sDP + ".Settings"))
	{
	iRes=dpGet(sDP + ".Settings.man_ConfigPath", dsParam[LASER_CONFIGPATH],
				sDP + ".Settings.man_ShowStatistics", dsParam[LASER_SHOWSTATISTICS],
				sDP + ".Settings.man_BackupPeriod", dsParam[LASER_BACKUP_PERIOD],
				sDP + ".Settings.tcp_Timeout", dsParam[LASER_TCP_TIMEOUT],
				sDP + ".Settings.tcp_LocalHost", dsParam[LASER_TCP_LOCALHOST],
				sDP + ".Settings.tcp_BrokerName", dsParam[LASER_TCP_REMOTEHOST],
				sDP + ".Settings.tcp_BrokerPort", dsParam[LASER_TCP_REMOTEPORT],
				sDP + ".Settings.tcp_TopicBase", dsParam[LASER_TCP_DATAPATH],
				sDP + ".Settings.tcp_MaxPacketSize", dsParam[LASER_TCP_MAXPACKETSIZE],
//				sDP + ".Settings.tcp_TestMode", dsParam[LASER_TCP_TESTMODE],
//				sDP + ".Settings.man_Name", dsParam[LASER_NAME],
				sDP + ".Settings.man_NumPeriod", dsParam[LASER_NUM_PERIOD],
				sDP + ".Settings.man_BufferPeriod", dsParam[LASER_BUFFER_PERIOD],
				sDP + ".Settings.man_StatisticsType", dsParam[LASER_TYPESTATISTICS],
      sDP + ".Settings.T_mode", dsParam[LASER_T_MODE]);

	if (iRes<0)
		fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_LoadSettingsManager(): " + getCatStr(g_sCat_Laser,"DPGETERROR"),"");
			
	}
else
	fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_LoadSettingsManager(): " + getCatStr(g_sCat_Laser,"DPDOESNOTEXIST"),"");
	
}


// unLaser_getStatus
/**
Purpose: get state of Laser system

Parameter: 	sDp, string, input, DataPoint
			sName, string, output, Name of Manager
			iState, int, output, State of Manager
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unLaser_getStatus(int iIndex, int &iState)
{
int iState_1=0, iState_2=0;

iState=0;

if (dynlen(g_diLaser_PVSS)>=iIndex && dynlen(g_diLaser_Message)>=iIndex && iIndex>0) 
	{
	//PVSS
	if ((g_diLaser_PVSS[iIndex]!=1)) 														//Archive or Manager stopped
		iState_1=1;

	//Message
	if ((g_diLaser_Message[iIndex]>10001) && (g_diLaser_Message[iIndex]!=10150))		//10150 -> Manager Internal Purpose
		iState_2=g_diLaser_Message[iIndex];																			//Warning																									
	}

if (dynContains(g_diLaserCritical_Alarm,iState_2)>0 || (iState_1==1))	//Error(s)
	iState=c_Error;
else
	{
	if (iState_2>0)	
		iState=c_Warning;
	}
}


// unLaser_setStatusTable
/**
Purpose: set state of Laser system

Parameter: 	bType, bool, input, Manager or Archive
			sName, string, input, Name of Manager or Archive 
			iIndex, int, input, index for Global Variable 
			iState, int, input, Status
			tState_6, int, input, Time of Status change
			dbTimeout, dyn bool, input, True = Warning Timeout or False = Ok
						
Usage: Public

PVSS manager usage: Ctrl, UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_setStatusTable(string sName, int iIndex, int iState, time tState_6)
{
int i, length;
string sContent, sTable, sText_PVSS, sText_Command, sText_StateOut, sText_Message, sColor, t, T_mode_icon="";


	length=ManagersList.lineCount();
	sTable="ManagersList";
	
for (i=0;i<length;i++)
	{
	getValue(sTable, "cellValueRC", i, "Names", sContent);
	if (sContent==sName)			//find Line
		{
		unCoreLHCServices_PVSS(g_diLaser_PVSS[iIndex], sText_PVSS);
		sText_Command=getCatStr(g_sCat_Laser, g_diLaser_Command[iIndex]);
		sText_StateOut = getCatStr(g_sCat_Laser, g_diLaser_State[iIndex]);
		
		if (g_diLaser_Message[iIndex]!=10150)		//->internal purpose for WCCOALaser
			sText_Message = getCatStr(g_sCat_Laser, g_diLaser_Message[iIndex]);
		else
			getValue(sTable, "cellValueRC", i, "MessageOut", sText_Message);
				
		switch(iState)
			{
			case 0:
				sColor="unLaser_Ok";
				break;
			
			case c_Warning:
				sColor="unLaser_Warning";
				break; 
			
			case c_Error:
				sColor="unLaser_Bad";
				break;
			
			default:
				sColor="unLaser_Bad";
			}
    
    if(g_diLaser_T_mode[iIndex]==1)
    {
      sColor="unLaser_Bad";      
      T_mode_icon="T_mode";
      sText_Message="T Mode - "+sText_Message;
    }
		
		if (tState_6!="")
			{	
			t=formatTime("%c", tState_6);
	 	
	 		if (_UNIX)
				t+=timeFromGMT(); // ???
 			
 			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "X", T_mode_icon, sTable, "cellValueRC", i, "PVSS", sText_PVSS, sTable, "cellValueRC", i, "Command", sText_Command, sTable, "cellValueRC", i, "StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message, sTable, "cellValueRC", i, "Time", t);
 			}
 		else	
			setMultiValue (sTable, "cellBackColRC", i, "X", sColor, sTable, "cellValueRC", i, "X", T_mode_icon, sTable, "cellValueRC", i, "PVSS", sText_PVSS, sTable, "cellValueRC", i, "Command", sText_Command, sTable, "cellValueRC", i, "StateOut", sText_StateOut, sTable, "cellValueRC", i, "MessageOut", sText_Message);
		
		return;
		}
	}
}

// unLaser_Configuration
/**
Purpose: Initialize Laser configuration

Parameter: 	sFunction_Name, string, output, Function Name to be used to update Table for configFile
			sComponent_Name, string, output, Name of the Component used for this Manager
			iIntegrity_Timeout, int, output, Timeout to check if manager alive
			dsDefault_Settings, dyn_string, output, Return all the settings
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Configuration(dyn_string &dsDefault_Settings, string  &sFunction_Name, string &sComponent_Name, int &iIntegrity_Timeout, dyn_string &exceptionInfo_Temp)
{
	int iRes;
	string sSystemName =getSystemName();
	
	if (dpExists(sSystemName + "Laser_Info.defaultSettings"))
		{
		iRes=dpGet(sSystemName + "Laser_Info.defaultSettings.Settings", dsDefault_Settings, sSystemName + "Laser_Info.configFile.Function", sFunction_Name, sSystemName + "Laser_Info.configFile.ComponentName", sComponent_Name);
		if (iRes<0)
			fwException_raise(exceptionInfo_Temp, "ERROR", "unLaser_Configuration(): " + getCatStr(g_sCat_Laser,"DPSETGETFAILED"),"");	
		
		}
	else
		{
		sFunction_Name="get_UnicosInfos";
		sComponent_Name="Unicos";
		iIntegrity_Timeout=300;
		dsDefault_Settings[LASER_CONFIGPATH]="data/LaserData.xml";
		dsDefault_Settings[LASER_TCP_LOCALHOST]="localhost";
		dsDefault_Settings[LASER_TCP_REMOTEHOST]="laser-test:2580";
		dsDefault_Settings[LASER_TCP_DATAPATH]="EN_ICE_TEST";
	//	dsDefault_Settings[LASER_NAME]=sSystemName();
		dsDefault_Settings[LASER_SHOWSTATISTICS]="200";
		dsDefault_Settings[LASER_TYPESTATISTICS]="6";
		dsDefault_Settings[LASER_TCP_REMOTEPORT]="2580";
//		dsDefault_Settings[LASER_TCP_TESTMODE]="0";
   dsDefault_Settings[LASER_T_MODE]="FALSE";
		dsDefault_Settings[LASER_TCP_MAXPACKETSIZE]="1000";
		dsDefault_Settings[LASER_TCP_TIMEOUT]="30";
		dsDefault_Settings[LASER_BACKUP_PERIOD]="600";
		dsDefault_Settings[LASER_BUFFER_PERIOD]="2";
		dsDefault_Settings[LASER_NUM_PERIOD]="10"; 
		
		iRes=dpCreate("Laser_Info","_Un_Laser_Config");
		if (iRes<0)
			{
			fwException_raise(exceptionInfo_Temp, "ERROR", "unLaser_Configuration(): " + getCatStr(g_sCat_Laser,"DPCREATEFAILURE"),"");	
			}
		else
			{																																																																					
			iRes= dpSetWait(sSystemName + "Laser_Info.defaultSettings.Settings", dsDefault_Settings, sSystemName + "Laser_Info.configFile.Function", sFunction_Name, sSystemName + "Laser_Info.configFile.ComponentName", sComponent_Name);
			}		
		}
}

// unLaser_getPosition
/**
Purpose: get index for Manager (need for Global dyn)

Parameter: 	sName, string, input, Archive's name or Manager's name
			iIndex, int, output, position in Array of this name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_getPosition(string sName, int &iIndex)
{
int i, len;

	iIndex=0;
	len=dynlen(g_dsLaser_Name);
	for (i=1;i<=len;i++)
		{
		if (g_dsLaser_Name[i]==sName)
			{
			iIndex=i;
			i=len+1;
			}
		}		
}

// unLaser_Check
/**
Purpose: Check Line syntax for Config File

Parameter:	sDpe, string, input, string for the Dpe cell
			sName, string, input, string for the Name
			sState, string, input, string for the State
			sRange, string, input, string for the Range
			sFF, string, input, string for the Fault Family 
			sFM, string, input, string for the Fault Member 
			sFC, string, input, string for the Fault Code 
			sDescription, string, input, string for the Description
			sMess_Error, string, output, Pattern for the error 
			bError, bool, input, true=Error
			stable, string, input, Table name to update
			bPanelCheck, bool, input, true=Need to add line before setColor
			iRow, int, input, Line Number in Table
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Check(string stable, bool bPanelCheck, int iRow, string &sMess_Error, bool &bError, string Select, string Type,  string Alias,string Dpe, string Name, string Range, string State, string FF, string FM, string FC, string Description)
{
string sError;
int iLine;

	if (Type!="")
		g_sLastType=Type;
	
	if (Dpe!="")
		g_sLastDpe=Dpe;
		
	if (Select!="S")
	{
		unLaser_CheckLine(g_sLastType, Alias, Dpe, Name, Range, State, FF, FM, FC, Description, sError);

		if (sError!="")
		{
		// In case of just checking (unLaserConfigFile_check.pnl) -> need to add line in Table
			if (bPanelCheck)
			{
				setValue(stable,"appendLine",
						"Select", Select,
						"Type", g_sLastType, 
						"Alias", Alias,
						"Range", Range, 
						"Description", Description, 
						"Name", Name, 
						"Dpe", g_sLastDpe, 
						"State", State,
						"ff", FF,
						"fm", FM,
						"fc", FC);
				getValue(stable,"lineCount",iLine);
				iLine=iLine-1;
			}											
			else iLine=iRow;
			bError=true;
			
		switch(sError)
			{
			case "Dpe":						
				setMultiValue(	stable, "cellBackColRC", iLine, "Dpe", "unLaser_Bad");				
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKDPE") + "\n";
				if (!bPanelCheck)
				{
					getValue(stable, "cellValueRC", iRow + 1, "Dpe", Dpe);
					if (Dpe=="")
						{
						setMultiValue(	stable, "cellBackColRC", iLine, "Dpe", "unLaser_Bad");	
						}
				}
			break;
			case "Alias":						
				setMultiValue(	stable, "cellBackColRC", iLine, "Alias", "unLaser_Bad");				
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKDPE") + "\n";
			break;

			case "Name":			
				setMultiValue (stable, "cellBackColRC", iLine,"Name", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKNAME") + Name + "\n";
			break;
			
			case "Description":
				setMultiValue (stable, "cellBackColRC", iLine, "Description", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKDESCRIPTION") + Description + "\n";
			break;
			
			case "FF":
				setMultiValue (stable, "cellBackColRC", iLine, "ff", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKFF") + FF + "\n";
			break;
			
			case "FM":
				setMultiValue (stable, "cellBackColRC", iLine, "fm", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKFM") + FM + "\n";
			break;
			
			case "FC":
				setMultiValue (stable, "cellBackColRC", iLine, "fc", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKFC") + FC + "\n";
			break;
			
			case "State":
				setMultiValue (stable, "cellBackColRC", iLine, "State", "unLaser_Bad");
				sMess_Error= sMess_Error + "\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKSTATE") + State + "\n\n";
			break;
		
			case "Range":
				setMultiValue (stable, "cellBackColRC", iLine, "Range", "unLaser_Bad");
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKRANGE") + "\n";
			break;
		
			default:
				setMultiValue(	stable, "cellBackColRC", iLine, "Select", "unLaser_Bad", 
								stable, "cellBackColRC", iLine, "Type", "unLaser_Bad",
								stable, "cellBackColRC", iLine, "Alias", "unLaser_Bad", 
								stable, "cellBackColRC", iLine, "Range", "unLaser_Bad", 
								stable, "cellBackColRC", iLine,"Description", "unLaser_Bad", 
								stable, "cellBackColRC", iLine, "Name", "unLaser_Bad", 
								stable, "cellBackColRC", iLine, "Dpe", "unLaser_Bad", 
								stable, "cellBackColRC", iLine, "State", "unLaser_Bad",
								stable, "cellBackColRC", iLine, "ff", "unLaser_Bad",
								stable, "cellBackColRC", iLine, "fm", "unLaser_Bad",
								stable, "cellBackColRC", iLine, "fc", "unLaser_Bad",
								stable, "cellValueRC", iLine, "Select", "F");
			}
		}
		else
		{
			if ((Select=="Q")&&(Dpe!="")&&(Alias==""))
			{
				if (bPanelCheck)
				{
					setValue(stable,"appendLine",
							"Select", Select,
							"Type", Type, 
							"Alias", Alias, 
							"Range", Range, 
							"Description", Description, 
							"Name", Name, 
							"Dpe", Dpe, 
							"State", State,
							"ff", FF,
							"fm", FM,
							"fc", FC);
				
					getValue(stable,"lineCount",iLine);
					iLine=iLine-1;
					setMultiValue(stable, "cellBackColRC", iLine, "Dpe", "unLaser_NotDefined");
				}											
				bError=true;
				sMess_Error= sMess_Error + "\n\t-> " + g_sLastDpe + "\n";
				sMess_Error= sMess_Error + "\t" + getCatStr(g_sCat_Laser,"LOGCHECKNOTDEFINED") + "\n";
			}
		}					
	}
}

// unLaser_CheckLine
/**
Purpose: Check Line syntax for Config File

Parameter: 	iType, int, input, define Variable Type
			bAlarm, bool, input, true -> PVSS alarm; false -> No PVSS alarm
			sDpe, string, input, string for the Dpe cell
			sName, string, input, string for the Name
			sState, string, input, string for the State
			sFC, string, input, string for the Fault Code 
			sDescription, string, input, string for the Description
			sEr, string, output, Pattern for the error 
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_CheckLine(string sMode, string sAlias, string sDpe, string sName, string sRange, string sState, string sFF, string sFM, string sFC, string sDescription, string &sEr)
{
int iEr, iFC, iEr_2;
string sFC_Temp;
dyn_string dsSplitHierarchy;

	sEr="";
				if ((sDpe!="") && (sAlias=="")){
		//Case 1 : dpe="something" alias =
				if (!dpExists(sDpe)) sEr="Dpe";	
			}
		else 
			if ((sDpe!="") && (sAlias!="")){
		//Case 2 : dpe="ProcessInput.Post" alias ="Alias"
				sDpe = unGenericDpFunctions_dpAliasToName(sAlias)+sDpe;
				if (!dpExists(sDpe)) sEr="Dpe";	
			}
		
	//. Name	
	iEr=0;
	iEr=strpos(sName,"\"");
	iEr=strpos(sName,"'") + iEr;
	iEr=strpos(sName,">") + iEr;
	iEr=strpos(sName,"<") + iEr;
	iEr=strpos(sName,"~") + iEr; 
	if (iEr> -5 || sName=="ERROR")		
		{
		sEr="Name";
		return;
		}
	
	//. Range
	if (sRange=="ERROR")		
		{
		sEr="Range";
		return;
		}
	
	//. State
	switch(sMode)
		{
		case "[BOOLEAN]":
			if (sState!="TERMINATE" && sState!="ACTIVE" && !patternMatch("*PVSS ALARM*",sState))
				{
				sEr="State";
				return;
				}
			break;
					
		case "[INTEGER]":
			sEr="State";
			return;
			break;
		
		case "[UNSIGNED]":
			sEr="State";
			return;
			break;
		
		case "[FLOAT]":
			if (sState!="TERMINATE" && sState!="ACTIVE" && !patternMatch("*PVSS ALARM*",sState))
				{
				sEr="State";
				return;
				}
			break;
		
		default:
			sEr="State";
			return;
		}
	
	//. FF
	if (sFF=="ERROR" || strlen(sFF)<1)		
		{
		sEr="FF";
		return;
		}
//	DebugN("sFF",sFF);
	
	//. FM
	if (sFM=="ERROR" || strlen(sFM)<1)		
		{
		sEr="FM";
		return;
		}
	
	//. FC
	iEr=sscanf(sFC, "%d", iFC); 
	iEr_2=sprintf(sFC_Temp, "%d", iFC);
	if (iEr<1 || strlen(sFC)!=strlen(sFC_Temp))	//Not Numeric Format
		{
		sEr="FC";
		return;
		}
	
	//. Description
	iEr=0;
	iEr=strpos(sDescription,"\"");
	iEr=strpos(sDescription,"'") + iEr;
	iEr=strpos(sDescription,">") + iEr;
	iEr=strpos(sDescription,"<") + iEr;
	iEr=strpos(sDescription,"~") + iEr; 
	
	if (iEr> -5 || sDescription=="ERROR")		
		sEr="Description";

}

// unLaser_getFromFile
/**
Purpose: Check Line syntax for Config File

Parameter: 	sConfigPath, string, input, Path
			Selected, dyn_string, output, array for the Selected column 
			Archive, dyn_string, output, array for the Archive column  
			Alias, dyn_string, output, array for the Alias column 
			Dpe, dyn_string, output, array for the Dpe column 
			Name, dyn_string, output, array for the Name column
			Description, dyn_string, output, array for the Description column
			Range, dyn_string, output, array for the Range column
			State, dyn_string, output, array for the State column
			FF, dyn_string, output, array for the FF column
			FM, dyn_string, output, array for the FM column
			FC, dyn_string, output, array for the FC column
			
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/
unLaser_getFromFile(string sConfigPath, dyn_string &Select, dyn_string &Type,  dyn_string &Alias, dyn_string &Dpe, dyn_string &Name, dyn_string &Description, dyn_string &Range, dyn_string &State, dyn_string &FM, dyn_string &FF, dyn_string &FC, dyn_int &Line, bool &bEr)
{
string fileInString, sSelect, sType, sDescription, sName, sDpe, sFC, sFM, sFF, sRange, sState, sPath, sAlias;
int i, len, iLen_Range,imaxRange,j,len2,iNbLine,iStart ;
int c_DESCRIPTION=2, c_FF=3, c_FM=4, c_FC=5, c_STATE=6;
float fMax, fIndex;
dyn_string dsFileLines, dsSplitRange;
dyn_dyn_string ddsFile;
bool bChange, bDefined;

		iNbLine=0;
	if (isFunctionDefined("ctrlConfRead"))
	{
		ctrlConfRead(sConfigPath,ddsFile);
		len=dynlen(ddsFile);
		fMax=len;
		for (i=1;i<=len;i++) //read each statement
		{				
			int l=1;
			dsFileLines=ddsFile[i];
			len2=dynlen(dsFileLines);
			fIndex=i;

			progressBar.progress(fIndex/fMax*100);
			if(patternMatch("*[*]*", dsFileLines[l]))
			{
				switch (dsFileLines[l])
					{
						case "[BOOLEAN]":
							sType=dsFileLines[l];
							bDefined=true;
						break;
						case "[FLOAT]":
							sType=dsFileLines[l];
							bDefined=true;
						break;
						case "[INTEGER]":
							bDefined=false;
						break;
						case "[UNSIGNED]":
							bDefined=false;
						break;
						default:
						bDefined=false;
					}//End switch		
						
					if(len2!=1)
						{
							dynAppend(Select,"F");
							dynAppend(Type,sType);
							dynAppend(Dpe,"ERROR");
							dynAppend(Alias,"ERROR");
							dynAppend(Line,iNbLine);		
							dynAppend(Description,"ERROR");
							dynAppend(Range,"ERROR");
							dynAppend(State,"ERROR");
							dynAppend(FF,"ERROR");
							dynAppend(FC,"ERROR");
							dynAppend(FM,"ERROR");
							dynAppend(Name,"ERROR");
							bEr=true;
						}
				iNbLine++;
			} //End pattern type
			else //dpe
			{
				if((bDefined)&(dsFileLines[l]==""))
				{
					iNbLine++;
					l++;
					if(patternMatch("*dpe=*", dsFileLines[l]))
					{
						sDpe=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
						l++;
						iNbLine++;
						if(patternMatch("*alias=*", dsFileLines[l]))
						{
							sAlias=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
							l++;			
							if(patternMatch("*name=*", dsFileLines[l]))
							{
								sName=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
								l++;						
								sSelect="X";
							} //End pattern name
							else
							{
								sSelect="F";
								sName="ERROR";
								bEr=true;
								if(!patternMatch("*range=*", dsFileLines[l])) l++;
							}			
						} //End pattern alias
						else
						{
							sSelect="F";
							sAlias="ERROR";
							bEr=true;
						}
						
					}// End pattern dpe
					else //dpe ERROR
					{
						sSelect="F";
						sDpe="ERROR";
						sAlias="ERROR";
						sName="ERROR";
						bEr=true;	
					}
						dynAppend(Name,sName);
						dynAppend(Type,sType);
						dynAppend(Alias,sAlias);
						dynAppend(Dpe,sDpe);
					if (sType == "[FLOAT]")	iStart=1;
						else iStart=0;
						
					for(j=0;j<=len2-l;j++)
							{						
							iNbLine++;
							int temp = j+iStart;
								if(patternMatch("*range="+temp+"*", dsFileLines[j+l]))
								{
									dsSplitRange=strsplit(dsFileLines[j+l], ";");
									iLen_Range=dynlen(dsSplitRange);
									
									if (iLen_Range>=5)	
									{
										if(j>0)
										{	
											dynAppend(Type,"");
											dynAppend(Alias,"");
											dynAppend(Dpe,"");
											dynAppend(Name,"");
											
										}

										dynAppend(Line,iNbLine);
										dynAppend(Select,sSelect);
										sDescription= substr(dsSplitRange[c_DESCRIPTION], strpos(dsSplitRange[c_DESCRIPTION],"=")+2,strlen(dsSplitRange[c_DESCRIPTION])-15);
										sFF= substr(dsSplitRange[c_FF], strpos(dsSplitRange[c_FF],"=")+1,strlen(dsSplitRange[c_FF]));
										sFM= substr(dsSplitRange[c_FM], strpos(dsSplitRange[c_FM],"=")+1,strlen(dsSplitRange[c_FM]));
										sFC= substr(dsSplitRange[c_FC], strpos(dsSplitRange[c_FC],"=")+1,strlen(dsSplitRange[c_FC]));
										if (iLen_Range>=c_STATE)
										{
											sState = substr(dsSplitRange[c_STATE], strpos(dsSplitRange[c_STATE],"=")+1,strlen(dsSplitRange[c_STATE]));
										}
										else
										{
											if(sType == "[FLOAT]") sState = "Analog PVSS ALARM";
											if(sType == "[BOOLEAN]")  sState = "Binary PVSS ALARM";	
										}
										dynAppend(Range,temp);
										dynAppend(Description,sDescription);
										dynAppend(FF,sFF);
										dynAppend(FC,sFC);
										dynAppend(FM,sFM);
										dynAppend(State,sState);
									} //end if iLen_Range >=5
									else
									{
										if(j>0)
										{	
											dynAppend(Type,"");
											dynAppend(Alias,"");
											dynAppend(Dpe,"");
											dynAppend(Name,"");
										}
										dynAppend(Description,"ERROR");
										dynAppend(Range,"ERROR");
										dynAppend(State,"ERROR");
										dynAppend(FF,"ERROR");
										dynAppend(FC,"ERROR");
										dynAppend(FM,"ERROR");
										bEr=true;
									}
								}//End range j
								else
								{
									if(j>0)
									{	
										dynAppend(Type,"");
										dynAppend(Alias,"");
										dynAppend(Dpe,"");
										dynAppend(Name,"");
									}
									dynAppend(Line,iNbLine);
									dynAppend(Description,"ERROR");
										dynAppend(Range,"ERROR");
										dynAppend(State,"ERROR");
										dynAppend(FF,"ERROR");
										dynAppend(FC,"ERROR");
										dynAppend(FM,"ERROR");
										dynAppend(Select,"F");
									bEr=true;
								}//End range error
								
							}//End for
				}
				else //first element error
				{
					dynAppend(Select,"F");
					dynAppend(Type,"ERROR");
					dynAppend(Alias,"ERROR");
					dynAppend(Dpe,"ERROR");
					dynAppend(Line,iNbLine);		
					dynAppend(Description,"ERROR");
					dynAppend(Range,"ERROR");
					dynAppend(State,"ERROR");
					dynAppend(FF,"ERROR");
					dynAppend(FC,"ERROR");
					dynAppend(FM,"ERROR");
					dynAppend(Name,"ERROR");
					bEr=true;
				}
			} //End else pattern type
		} //End for
	}//End if function defined
	else
	{
	DebugN("DLL not defined");
	bEr=true;
	}//End function not defined
	
}

// unLaser_LoadDpType
/**
Purpose: Load DpType list to execute query

Parameter: 	sType, string, input, Variable type
			dsSplit, dyn_string, output, DpType list for this variable type
			bInternal, bool, input, true means that we want to see Internal Dp ("_*")
			exceptionInfoTemp, dyn_string, output, Error message
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_LoadDpType(string sType, dyn_string &dsSplit, bool bInternal, bool &bFind, dyn_string &exceptionInfoTemp)
{
dyn_string dsType, dsTempSplit;
string sSystemName=getSystemName();
int i, j, iRes, length, length_2;

	if (sType!="")
		{
		if (dpExists(sSystemName + "Laser_Info.configFile"))
			{
			iRes=dpGet(sSystemName + "Laser_Info.configFile.DpType", dsType);
			
			if (iRes < 0)
				{
				fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_LoadDpType(): " + getCatStr(g_sCat_Laser,"DPSETGETFAILED"), "");
				return;
				}
			g_bEdit=false;
			}
		else
			{
			fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_LoadDpType(): " + getCatStr(g_sCat_Laser,"DPDOESNOTEXIST"), "");
			return;
			}
		
		length=dynlen(dsType);	
		dynClear(dsSplit);	
		for (i=1;i<=length;i++)				//For each variable Type
			{
			dsSplit=strsplit(dsType[i],";");
			
			if (dynlen(dsSplit)>1)			//At leat one dpType defined
				{
				if (dsSplit[1]==sType)
					{
					bFind=true;
					i=length+1;
					dynRemove(dsSplit,1);		//Remove Variable Type
					
					if (!bInternal)
						{
						length_2=dynlen(dsSplit);
						dynClear(dsTempSplit);	
						dsTempSplit=dsSplit;
						for (j=1;j<=length_2;j++)
							{
							if (strpos(dsSplit[j],"_")==0)
								dynRemove(dsTempSplit,j);		//Remove Internal DP
							}
						dynClear(dsSplit);	
						dsSplit=dsTempSplit;
						}
					}
				else
					bFind=false;
				}
			else
				bFind=false;
			}
			
		if (!bFind)
			dynClear(dsSplit);
		}
}

// unLaser_Query
/**
Purpose: create the Query

Parameter: 	dsDpeNames, dyn_string, output, Dpe return
			diType, dyn_string, output, alert_hdl Type
			exceptionInfoTemp, dyn_string, output, Error message(s)
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Query(dyn_string &dsAliasNames, dyn_string &dsDpeNames, dyn_string &diType, dyn_string &exceptionInfoTemp)
{
bool bDefined;
int i, j, length, size, iRes,k;
float fIndex, fMax;
string selectQuery, sWhere="WHERE ", sDPT="_DPT=\"", sELC="_ELC=", sAnd=" AND (", sOr="\" OR ", sQuery;
dyn_string dsDpType, dsQuery;
dyn_string dsLaser_VariableType=makeDynString("DPEL_BOOL","DPEL_FLOAT","DPEL_INT","DPEL_UINT");
dyn_int diLaser_VariableType=makeDynInt(23,22,21,20);
dyn_dyn_anytype tabResult, dstabResultTemp;

	dynClear(tabResult); 
	k=1;
	selectQuery = "SELECT '_alert_hdl.._type' FROM '*' ";
	size=dynlen(dsLaser_VariableType);
	fMax=size;
	for (j=1;j<=size;j++)					//for each variable type
	{
		fIndex=size - j;
		progressBar.progress(fIndex/fMax*100);
		
		unLaser_LoadDpType(dsLaser_VariableType[j], dsDpType, true, bDefined, exceptionInfoTemp);
		if (dynlen(exceptionInfoTemp)==0)
		{	
			if (bDefined)
			{
				selectQuery = selectQuery + sWhere + sELC + dsLaser_VariableType[j] + sAnd;
				length=dynlen(dsDpType);
				for (i=1;i<=length;i++)			//for each DpType
				{
					if (i==1)
						sQuery  = sQuery + sDPT + dsDpType[i];
					else
						sQuery  = sQuery + sOr + sDPT + dsDpType[i];
				}
				sQuery = sQuery + "\")";
				dsQuery[k]=selectQuery + sQuery;
				selectQuery = "SELECT '_alert_hdl.._type' FROM '*' ";
				sQuery = "";
				k++;
			}
		}
	}
	for (j=1;j<=dynlen(dsQuery);j++)					//for each variable type
	{
		iRes=dpQuery(dsQuery[j], dstabResultTemp);
		dynRemove(dstabResultTemp,1);						//remove first line which is not relevant
		dynAppend(tabResult,dstabResultTemp);
	}
	dynUnique(tabResult);
	if (iRes!=0)
		fwException_raise(exceptionInfoTemp, "ERROR", "unLaser_Query(): " + getCatStr(g_sCat_Laser,"DPQUERYFAILED") + selectQuery, "");
	else	
	{
		length = dynlen(tabResult);
		fMax=length;
		dynClear(dsDpeNames); 
		dynClear(diType);
		dyn_string dsSplit;
		for (i = 1; i <= length; i++)   
		{
			fIndex=j;
			progressBar.progress(fIndex/fMax*100);
			dynAppend(dsDpeNames, tabResult[i][1]);
			dynAppend(diType, tabResult[i][2]);
		}

		length = dynlen(dsDpeNames);
		for (i = 1; i <= length; i++)   
		{
			fIndex=j;
			progressBar.progress(fIndex/fMax*100);
			dsSplit=strsplit(dsDpeNames[i],".");
			if (dynlen(dsSplit)==3)
			{
				dynAppend(dsAliasNames, dpGetAlias(dsSplit[1] + "."));
			}
				else 	dynAppend(dsAliasNames, "");
		}
	}
}

// unLaser_Update_PVSS_Status
/**
Purpose: Start 
Parameter: 	sManager, string, input, Manager Name
			diState_PVSS, dyn_string, States of Manager in PVSS console
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Update_PVSS_Status(string sManager, int iState_PVSS)
{
int iPosition, iStatus, iRes;
dyn_string exceptionInfoTemp;

	unLaser_getPosition(sManager, iPosition);

	if (iState_PVSS!=g_diLaser_PVSS[iPosition])			//PVSS Status has changed
		{
		g_diLaser_PVSS[iPosition]=iState_PVSS;
		
		if (g_bPanel)		
			{//2.2 Get State
			unLaser_getStatus(iPosition, iStatus);
			
			//2.3 Update Table in Panel
			unLaser_setStatusTable(sManager, iPosition, iStatus, "");	
			}
		else
			{
			iRes=dpSet(getSystemName() + sManager + ".State.ev_MessageOut", g_diLaser_Message[iPosition]);
			}		
		}
}

// unLaser_Integrity
/**
Purpose: Start 
Parameter: 	sManager, string, input, Manager Name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Integrity(string sManager, string sSystem)
{
int iPosition;

	unLaser_getPosition(sManager, iPosition);
	if (g_diLaser_Period[iPosition]<=0)
		g_diLaser_Period[iPosition]=c_Min_Rate;
	
	while (true) 	
		{
	  	unLaser_Check_Integrity(sManager, sSystem);
	  	unLaser_getPosition(sManager, iPosition);
	  	if (iPosition<1)
			delay(c_Min_Rate);
	  	else 
			delay(g_diLaser_Period[iPosition]); 
		} 
}

// unLaser_Check_Integrity
/**
Purpose: Function checks period of answer from Manager

Parameter: 	sMan, string, input, Manager Name
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_Check_Integrity(string sMan, string sSystem)
{
int iLastRefresh, iNow, iPeriod, iRes, iPosition, iMan_Num, iManager;
time tNow, tLaser_LastRefresh;
//string sSystem, sDatapoint_1, sDatapoint_2;
string sDatapoint_1, sDatapoint_2;
dyn_string exceptionInfo;

	tNow=getCurrentTime();
	unLaser_getPosition(sMan, iPosition);

	sDatapoint_1 = sSystem + sMan + ".Settings.man_Num";
	iRes=dpGet(sDatapoint_1, iMan_Num);

	if (iRes>=0)
		{
		if(dynContains(g_diLaser_Api, iMan_Num))
			{
			sDatapoint_2 = sSystem + sMan + ".Settings.man_Num:_online.._stime";
			iRes=dpGet(sDatapoint_2, tLaser_LastRefresh);
			if (iRes>=0)
				{
				//Get Time in seconds from 1970
				if (isFunctionDefined("ctrlTime"))
					{
					iNow=ctrlTime(tNow);		
					iLastRefresh=ctrlTime(tLaser_LastRefresh);
					}
				else
					{
					iNow=0;		
					iLastRefresh=0;
					fwException_raise(exceptionInfo, "ERROR", "unLHCLogging_Save_Delay_Archive(): " + getCatStr(g_sCat_Laser,"DLLERROR"),"");
					}

				if (iNow!=0 && iLastRefresh!=0)
					iPeriod=iNow-iLastRefresh;		
				else
					iPeriod=g_diLaser_Period[iPosition] + 1;	//Error
				
				//DebugN(iPeriod + " >? " + g_diLaser_Period[iPosition]);
				if (iPeriod > g_diLaser_Period[iPosition])
					iManager=2;		//blocked
				else
					iManager=1;		//Started
				}
			else
				iManager=-1;		//?
			}
		else
			iManager=0;				//Stopped
		}
	else
		iManager=-1;				//?
			
	unLaser_Update_PVSS_Status(sMan, iManager);	
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity__Laser_CB
/**
Purpose:
Get Global Information

	@param sDp_1: string, input, data point name
	@param iPeriod: int, Period in minutes where Laser updates man_Num

Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/


unSystemIntegrity__Laser_CB(string sDp_1, int iPeriod)
{
string sManager;
int iPosition;

	sManager=substr(sDp_1, strpos(sDp_1, c_Laser_Pattern), strlen(c_Laser_Pattern) + 2);
	
	unLaser_getPosition(sManager, iPosition);
	if (iPeriod<(c_Min_Rate/2))
		g_diLaser_Period[iPosition]=c_Min_Rate;
	else
		g_diLaser_Period[iPosition]= iPeriod * 2;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity__Laser_CB_Api
/**
Purpose:
Get Api Connections

	@param sDp_1: string, input, data point name
	@param diApi: dyn_int, Api Connections
Usage: Internal

PVSS manager usage: CTRL (WCCOACTRL)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unSystemIntegrity__Laser_CB_Api(string sDp_1, dyn_int diApi)
{
	g_diLaser_Api=diApi;
}


// -------------------------------------------------------------------------------------
// unLaser_parse4XMLconversion
/**
Purpose:
Parse old txt configuration file and return the elements, similar with the unLaser_getFromFile function but with less restrictions.

Parameter: 	sConfigPath, string, input, Path
			Selected, dyn_string, output, array for the Selected column 
			Archive, dyn_string, output, array for the Archive column  
			Alias, dyn_string, output, array for the Alias column 
			Dpe, dyn_string, output, array for the Dpe column 
			Name, dyn_string, output, array for the Name column
			Description, dyn_string, output, array for the Description column
			Range, dyn_string, output, array for the Range column
			State, dyn_string, output, array for the State column
			FF, dyn_string, output, array for the FF column
			FM, dyn_string, output, array for the FM column
			FC, dyn_string, output, array for the FC column
			
						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

unLaser_parse4XMLconversion(string sConfigPath, dyn_string &Select, dyn_string &Type,  dyn_string &Alias, dyn_string &Dpe, dyn_string &Name, dyn_string &Description, dyn_string &Range, dyn_string &State, dyn_string &FM, dyn_string &FF, dyn_string &FC, dyn_int &Line, dyn_string &Limit, bool &bEr)
{
  string fileInString, sSelect, sType, sDescription, sName, sDpe, sFC, sFM, sFF, sRange, sState, sPath, sAlias, sLimit;
  int i, len, iLen_Range,imaxRange,j,len2,iNbLine,iStart ;
  int c_DESCRIPTION=2, c_FF=3, c_FM=4, c_FC=5, c_STATE=6, c_Limit=7;
  float fMax, fIndex;
  dyn_string dsFileLines, dsSplitRange;
  dyn_dyn_string ddsFile;
  bool bChange, bDefined;
  
  //string ERROR_VALUE="!ERROR_VALUE!";
  string ERROR_VALUE="";
  progressBar.visible(true);
  iNbLine=0;	
  
  if (isFunctionDefined("ctrlConfRead"))
  {
    ctrlConfRead(sConfigPath,ddsFile);
		len=dynlen(ddsFile);
		fMax=len;
		for (i=1;i<=len;i++) //read each statement
		{				
			int l=1;
			dsFileLines=ddsFile[i];
			len2=dynlen(dsFileLines);
			fIndex=i;

			progressBar.progress(fIndex/fMax*100);
			if(patternMatch("*[*]*", dsFileLines[l]))
			{
				switch (dsFileLines[l])
					{
						case "[BOOLEAN]":
							sType=dsFileLines[l];
							bDefined=true;
						break;
						case "[FLOAT]":
							sType=dsFileLines[l];
							bDefined=true;
						break;
						case "[INTEGER]":
						//bDefined=false;
              sType=dsFileLines[l];
              bDefined=true;              
						break;
						case "[UNSIGNED]":
              sType=dsFileLines[l];
              bDefined=true; 
						break;
						default:
						bDefined=false;
					}//End switch		
						
					if(len2!=1)
						{
							dynAppend(Select,"F");
							dynAppend(Type,sType);
							dynAppend(Dpe,ERROR_VALUE);
							dynAppend(Alias,ERROR_VALUE);
							dynAppend(Line,iNbLine);		
							dynAppend(Description,ERROR_VALUE);
							dynAppend(Range,ERROR_VALUE);
							dynAppend(State,ERROR_VALUE);
							dynAppend(FF,ERROR_VALUE);
							dynAppend(FC,ERROR_VALUE);
							dynAppend(FM,ERROR_VALUE);
							dynAppend(Name,ERROR_VALUE);
							bEr=true;
						}
				iNbLine++;
			} //End pattern type
			else //if(dsFileLines[l]!="")
			{
				if((bDefined)&(dsFileLines[l]==""))
				{
					iNbLine++;
					l++;
					if(patternMatch("*dpe=*", dsFileLines[l]))
					{
						sDpe=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
						l++;
						iNbLine++;
						if(patternMatch("*alias=*", dsFileLines[l]))
						{
							sAlias=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
							l++;			
							if(patternMatch("*name=*", dsFileLines[l]))
							{
								sName=substr(dsFileLines[l], strpos(dsFileLines[l],"=")+1,strlen(dsFileLines[l]));
								l++;						
								sSelect="X";
							} //End pattern name
							else
							{
								sSelect="F";
								sName=ERROR_VALUE;
								bEr=true;
								if(!patternMatch("*range=*", dsFileLines[l])) l++;
							}			
						} //End pattern alias
						else
						{
							sSelect="F";
							sAlias=ERROR_VALUE;
							bEr=true;
						}
						
					}// End pattern dpe
					else //dpe ERROR
					{
						sSelect="F";
						sDpe=ERROR_VALUE;
						sAlias=ERROR_VALUE;
						sName=ERROR_VALUE;
						bEr=true;	
					}
						dynAppend(Name,sName);
						dynAppend(Type,sType);
						dynAppend(Alias,sAlias);
						dynAppend(Dpe,sDpe);
					if (sType == "[FLOAT]" || sType == "[INTEGER]" || sType == "[UNSIGNED]")	iStart=1;
						else iStart=0;
						
					for(j=0;j<=len2-l;j++)
							{						
							iNbLine++;
							int temp = j+iStart;
								if(patternMatch("*range="+temp+"*", dsFileLines[j+l]))
								{
									dsSplitRange=strsplit(dsFileLines[j+l], ";");
									iLen_Range=dynlen(dsSplitRange);
                  makeDynString("abc", "cdea", "abcd", "cdeafgh", "bbacc", "xyz");
									
									if (iLen_Range>=5)	
									{
                    //tha prepei na elenksw oti yparxoun ta range=;description=;ff=;fm=;fc=;
                    //meta an einai ta range 6 + na eleksw gia gia state
                    // an megalitera tou 7 + limit!
                    
                    
										if(j>0)
										{	
											dynAppend(Type,"");
											dynAppend(Alias,"");
											dynAppend(Dpe,"");
											dynAppend(Name,"");											
										}
                    
                    
                    
                    

										dynAppend(Line,iNbLine);
										dynAppend(Select,sSelect);
										sDescription= substr(dsSplitRange[c_DESCRIPTION], strpos(dsSplitRange[c_DESCRIPTION],"=")+2,strlen(dsSplitRange[c_DESCRIPTION])-15);
										sFF= substr(dsSplitRange[c_FF], strpos(dsSplitRange[c_FF],"=")+1,strlen(dsSplitRange[c_FF]));
										sFM= substr(dsSplitRange[c_FM], strpos(dsSplitRange[c_FM],"=")+1,strlen(dsSplitRange[c_FM]));
										sFC= substr(dsSplitRange[c_FC], strpos(dsSplitRange[c_FC],"=")+1,strlen(dsSplitRange[c_FC]));
										if (iLen_Range>=c_STATE)
										{
											sState = substr(dsSplitRange[c_STATE], strpos(dsSplitRange[c_STATE],"=")+1,strlen(dsSplitRange[c_STATE]));
                    if (iLen_Range>=c_Limit) //If state is specified check if any limit is specified
                    {
                      sLimit = substr(dsSplitRange[c_Limit], strpos(dsSplitRange[c_Limit],"limit")+5,strlen(dsSplitRange[c_Limit]));
                      //DebugTN(sLimit,substr(dsSplitRange[c_Limit], strpos(dsSplitRange[c_Limit],"limit"),strlen(dsSplitRange[c_Limit])));
                      
                    }
                    //else
                    //  dynAppend(Limit,ERROR_VALUE); bEr=true;
										}
										else
										{
											if(sType == "[FLOAT]" || sType == "[INTEGER]" || sType == "[UNSIGNED]") sState = "";
											if(sType == "[BOOLEAN]")  sState = "";	sLimit = "";
										}
										dynAppend(Range,temp);
										dynAppend(Description,sDescription);
										dynAppend(FF,sFF);
										dynAppend(FC,sFC);
										dynAppend(FM,sFM);                    
										dynAppend(State,sState);
										dynAppend(Limit,sLimit);                    
									} //end if iLen_Range >=5
									else
									{
										if(j>0)
										{	
											dynAppend(Type,"");
											dynAppend(Alias,"");
											dynAppend(Dpe,"");
											dynAppend(Name,"");
										}
										dynAppend(Description,ERROR_VALUE);
										dynAppend(Range,ERROR_VALUE);
										dynAppend(State,ERROR_VALUE);
										dynAppend(FF,ERROR_VALUE);
										dynAppend(FC,ERROR_VALUE);
										dynAppend(FM,ERROR_VALUE);
										dynAppend(Limit,ERROR_VALUE);                    
										bEr=true;
									}
								}//End range j
								else
								{
									if(j>0)
									{	
										dynAppend(Type,"");
										dynAppend(Alias,"");
										dynAppend(Dpe,"");
										dynAppend(Name,"");
									}
									dynAppend(Line,iNbLine);
									dynAppend(Description,ERROR_VALUE);
										dynAppend(Range,ERROR_VALUE);
										dynAppend(State,ERROR_VALUE);
										dynAppend(FF,ERROR_VALUE);
										dynAppend(FC,ERROR_VALUE);
										dynAppend(FM,ERROR_VALUE);
										dynAppend(Limit,ERROR_VALUE);                                        
										dynAppend(Select,"F");
									bEr=true;
								}//End range error
								
							}//End for
				}
				else //first element error
				{
					dynAppend(Select,"F");
					dynAppend(Type,ERROR_VALUE);
					dynAppend(Alias,ERROR_VALUE);
					dynAppend(Dpe,ERROR_VALUE);
					dynAppend(Line,iNbLine);		
					dynAppend(Description,ERROR_VALUE);
					dynAppend(Range,ERROR_VALUE);
					dynAppend(State,ERROR_VALUE);
					dynAppend(FF,ERROR_VALUE);
					dynAppend(FC,ERROR_VALUE);
					dynAppend(FM,ERROR_VALUE);
					dynAppend(Limit,ERROR_VALUE);                              
					dynAppend(Name,ERROR_VALUE);
					bEr=true;
				}
			} //End else pattern type
		} //End for
	}//End if function defined
	else
	{
	DebugN("DLL not defined");
	bEr=true;
	}//End function not defined
  progressBar.visible(false);
}


// unLaser_convertToXML
/**
Purpose: Converts the .txt configuration file to an XML configuration file

Parameter: 	xmlFilePath, string, input, Filepath
			Selected, dyn_string, input, array for the Selected column 
      Type, dyn_string, input, array for the Type column 
			Alias, dyn_string, input, array for the Alias column 
			Dpe, dyn_string, input, array for the Dpe column 
			Name, dyn_string, input, array for the Name column
			Description, input, output, array for the Description column
			Range, dyn_string, input, array for the Range column
			State, dyn_string, input, array for the State column
			FF, dyn_string, input, array for the FF column
			FM, dyn_string, input, array for the FM column
			FC, dyn_string, input, array for the FC column

      returns the new converted xml filepath.
      						
Usage: Public

PVSS manager usage: UI (WCCOANV) and UI extended (WCCOANG)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT, XP and Linux, but tested only under XP and Linux.
	. distributed system: yes.
*/

string unLaser_convertToXML(string xmlFilePath, dyn_string Selected, dyn_string Type, dyn_string Alias, dyn_string Dpe, dyn_string Name, dyn_string Description, dyn_string Range, dyn_string State, dyn_string FF, dyn_string FM, dyn_string FC, dyn_string Limit)
{
  int i,j;
  file xmlFile;
  string xmlAlias,xmlName,xmlDpe,xmlDesc,xmlFF,xmlFM,xmlFC,xmlState,xmlType,xmlDescription,xmlRange,xmlLimit;
  //DebugTN(dynlen(Limit),dynlen(State),dynlen(Range),dynlen(FF));
  //DebugTN(Type);
  if(getExt(xmlFilePath) !="")
    xmlFilePath = delExt(xmlFilePath);
  xmlFilePath+=".xml";
  
  //dyn_int xmlRange;
  //xmlFilePath=getPath(DATA_REL_PATH) + "/xmlfileTest.xml";
  xmlFile=fopen(xmlFilePath,"w"); // open for writing. If a file with the given name already exists, the new text will be appended at the end.
  //root xml file node.
  fputs("<laserXSD:configFile xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:laserXSD=\"http://en-dep.web.cern.ch/en-dep/structure/ICE/\">\n",xmlFile);
  for(i=1;i<=dynlen(Type);i++)
  {
    //DebugTN(Selected[i]);
    if(Selected[i] == "F")
      continue;
    //DebugTN(Selected[i]);
    if(patternMatch("*[*]*", Type[i]))
    {
      
      //Set DPe Attribute
      if(Dpe[i]=="")
        xmlDpe="";
      else      
        xmlDpe="Dpe=\"" + Dpe[i] + "\" ";

      //Set DP Alias Attribute
      if(Alias[i]=="")
        xmlAlias="";
      else
        xmlAlias= "Alias=\"" + Alias[i] + "\" ";      

      //Set DP Type Attribute
      xmlType=strrtrim(strltrim(Type[i],"["),"]"); //trim [,] chars frm the Type Value.
      if(xmlType=="")
        xmlType="";
      else
        xmlType= "Type=\"" + xmlType + "\" ";

      //Set DP Name Attribute     
      if(Name[i]=="")
        xmlName="";
      else
        xmlName= "Name=\"" + Name[i] + "\" "; 
      
     //Set Range Num Attribute     
      if(Range[i]=="")
        xmlRange="";
      else
        xmlRange= "Num=\"" + Range[i] + "\" "; 
      
     //Set Range Description Attribute
      //remove any quotes
      strreplace(Description[i], "\"", "");     
      if(Description[i]=="")
        xmlDescription="";
      else
        xmlDescription= "Description=\"" + Description[i] + "\" ";       
     
     //Set Range FF Attribute     
      if(FF[i]=="")
        xmlFF="";
      else
        xmlFF="FF=\"" + FF[i] + "\" "; 

      //Set Range FM Attribute     
      if(FM[i]=="")
        xmlFM="";
      else
        xmlFM="FM=\"" + FM[i] + "\" ";       
      
      //Set Range C Attribute     
      if(FC[i]=="")
        xmlFC="";
      else
        xmlFC="FC=\"" + FC[i] + "\" ";    

      //Set Range State Attribute     
      if(State[i]=="")
        xmlState="";
      else
        xmlState="State=\"" + State[i] + "\" ";              
      
      //Set Range Limit Attribute     
      if(Limit[i]=="")
      {
        xmlLimit="";
      }
      else
      {
        //DebugTN(Limit[i]);
        xmlLimit="Limit=\"" + Limit[i] + "\" ";        
      }
        
      //DebugTN(xmlType,xmlAlias,xmlName,xmlDpe,xmlDesc,xmlFF,xmlFM,xmlFC,xmlState);
      //fputs("<Datapoint>\n",xmlFile); 
      //fputs("<Datapoint Dpe=\"" + xmlDpe + "\" Type=\"" + xmlType + "\" Alias=\"" + xmlAlias + "\" Name=\"" + xmlName + "\" T_mode=\"false\">\n", xmlFile);      
      //fputs("<Range Num=\""+xmlRange+"\" Description=\""+xmlDescription+"\" FF=\""+xmlFF+"\" FM=\""+xmlFM+"\" FC=\""+xmlFC+"\" State=\""+xmlState+"\"",xmlFile);  
      //fputs("<Datapoint " + xmlDpe + " " + xmlType + " " + xmlAlias + " " + xmlName + " T_mode=\"false\">\n", xmlFile);      
      //fputs("<Range "+xmlRange+" "+xmlDescription+" "+xmlFF+" "+xmlFM+" "+xmlFC+" "+xmlState+" "+xmlLimit+"/>\n",xmlFile);  
      fputs("<Datapoint " + xmlDpe + xmlType + xmlAlias + xmlName + "T_mode=\"false\">\n", xmlFile);      
      fputs("<Range "+ xmlRange + xmlDescription + xmlFF + xmlFM + xmlFC + xmlState + xmlLimit+"/>\n",xmlFile);        
/*      
      if(xmlLimit!="")
        fputs(" Limit=\""+xmlLimit+"\"/>\n", xmlFile);
      else 
        fputs("/>\n", xmlFile);      
//      DebugTN(i,Type[i]);      
        */
      j=i;
      if (j!=dynlen(Type))//if last element dont continue
        j++; 
      else
        break;
      //DebugTN("j",j);
      while(Type[j]=="")
      {
        //DebugTN(xmlType,xmlAlias,xmlName,xmlDpe,xmlDesc,xmlFF,xmlFM,xmlFC,xmlState);
        if(Selected[j] == "F")
        {
          j++;
          continue;
        }        

     //Set Range Num Attribute     
      if(Range[j]=="")
        xmlRange="";
      else
        xmlRange= "Num=\"" + Range[j] + "\" "; 
      
     //Set Range Description Attribute
      strreplace(Description[j], "\"", "");          
      if(Description[j]=="")
        xmlDescription="";
      else
        xmlDescription= "Description=\"" + Description[j] + "\" ";       
     
     //Set Range FF Attribute     
      if(FF[j]=="")
        xmlFF="";
      else
        xmlFF="FF=\"" + FF[j] + "\" "; 

      //Set Range FM Attribute     
      if(FM[j]=="")
        xmlFM="";
      else
        xmlFM="FM=\"" + FM[j] + "\" ";       
      
      //Set Range C Attribute     
      if(FC[j]=="")
        xmlFC="";
      else
        xmlFC="FC=\"" + FC[j] + "\" ";    

      //Set Range State Attribute     
      if(State[j]=="")
        xmlState="";
      else
        xmlState="State=\"" + State[j] + "\" ";              
      
      //Set Range Limit Attribute     
      if(Limit[j]=="")
        xmlLimit="";
      else
      {
        //DebugTN(Limit[j]);        
        xmlLimit="Limit=\"" + Limit[j] + "\" ";
      }    
        
        
        fputs("<Range "+ xmlRange + xmlDescription + xmlFF + xmlFM + xmlFC + xmlState + xmlLimit+"/>\n",xmlFile);      
        /*
        fputs("<Range Num=\""+xmlRange+"\" Description=\""+xmlDescription+"\" FF=\""+xmlFF+"\" FM=\""+xmlFM+"\" FC=\""+xmlFC+"\" State=\""+xmlState+"\"",xmlFile);
        if(xmlLimit!="")
          fputs(" Limit=\""+xmlLimit+"\"/>\n", xmlFile);
        else 
          fputs("/>\n", xmlFile);
          */
        
      if (j!=dynlen(Type))//if last element dont continue
        j++; 
      else
        break;
      }
     fputs("</Datapoint>\n",xmlFile); 
     //i=j;
     //DebugTN(i,Type[i]);
     }
  }
  fputs("</laserXSD:configFile>\n",xmlFile); 
  fflush(xmlFile);  
  fclose(xmlFile); // close file
  //DebugTN(xmlFilePath);
  return xmlFilePath;
}

